@@ -198,6 +198,37 @@ class HdDB:
             logger.error(f"Error retrieving data from MotherDuck: {e}")
             raise QueryError(f"Error retrieving data from MotherDuck: {e}")
 
+    @attach_motherduck
+    def get_record_by_id(self, org: str, db: str, tbl: str, id: str) -> dict:
+        """
+        Retrieve a record by its ID from a specified table in Motherduck
+
+        :param org: Organization name
+        :param db: Database name
+        :param tbl: Table name
+        :param id: Record ID
+        :return: Dictionary containing the record data and fields, or null values if no data found
+        """
+        try:
+            data_query = f'SELECT * FROM "{org}__{db}"."{tbl}" WHERE rcd___id = ?'
+            record = self.execute(data_query, [id]).fetchone()
+
+            if record:
+                fields_query = f'SELECT * FROM "{org}__{db}".hd_fields WHERE tbl = ?'
+                fields = self.execute(fields_query, [tbl]).fetchdf()
+                fields_json = fields.to_dict(orient="records")
+
+                # Convert record tuple to a dictionary with column names as keys
+                column_names = [field["id"] for field in fields_json]
+                record_dict = dict(zip(column_names, record))
+
+                return {"data": record_dict, "fields": fields_json}
+            else:
+                return {"data": None, "fields": None}
+        except duckdb.Error as e:
+            logger.error(f"Error retrieving record from MotherDuck: {e}")
+            raise QueryError(f"Error retrieving record from MotherDuck: {e}")
+
     @attach_motherduck
     def drop_table(self, org: str, db: str, tbl: str):
         """
@@ -633,3 +664,25 @@ class HdDB:
         except duckdb.Error as e:
             logger.error(f"Error updating row in table {tbl}: {e}")
             raise QueryError(f"Error updating row in table {tbl}: {e}")
+
+    @attach_motherduck
+    def get_fields(self, org: str, db: str, tbl: Optional[str] = None) -> list:
+        """
+        Retrieve fields from the hd_fields table for a given table.
+
+        :param org: Organization name
+        :param db: Database name
+        :param tbl: Table name
+        :return: List of fields from hd_fields table
+        :raises QueryError: If there's an error fetching fields from MotherDuck
+        """
+        try:
+            query = f'SELECT fld___id, id, label, type FROM "{org}__{db}".hd_fields'
+            if tbl is not None:
+                query += ' WHERE tbl = ?'
+            result = self.execute(query, [tbl] if tbl is not None else None).fetchdf()
+
+            return { "data": json.loads(result.to_json(orient="records")) }
+        except duckdb.Error as e:
+            logger.error(f"Error fetching fields from hd_fields: {e}")
+            raise QueryError(f"Error fetching fields from hd_fields: {e}")
